/*  <lalVerbatim file="LALInspiralSetupCV">
Author: Sathyaprakash, B. S.
$Id: LALInspiralSetup.c,v 1.9 2001/12/06 15:15:00 dchurches Exp $
</lalVerbatim>  */

/*  <lalLaTeX>

\subsection{Module \texttt{LALInspiralSetup.c}}

Module to generate all the Taylor and Pade coefficients needed in 
waveform generation. 

\subsubsection*{Prototypes}
\vspace{0.1in}
\input{LALInspiralSetupCP}
\idx{LALInspiralSetup()}

\subsubsection*{Description}

Module to generate all the coefficiants needed in the Taylor and Pade expansions of the energy and
flux functions $E^{\prime}(v)$ and $\mathcal{F}(v)$. 
These are used to solve the gravitational wave phasing formula,

\begin{eqnarray}
t(v) & = & t_{0} - m \int_{v_{0}}^{v} \,
\frac{E'(v)}{{\cal F}(v)} \, dv, \nonumber \\
\phi (v) & = & \phi_{0} - 2 \int_{v_{0}}^{v}  v^3 \,
\frac{E'(v)}{{\cal F}(v)} \, dv,
\end{eqnarray}

where $v=(\pi m F)^{1/3}$ is an invariantly defined velocity, $F$ is the instantaneous GW
frequency, and
$m$ is the total mass of the binary.

The expressions for $E^{\prime}(v)$ and $\mathcal{F}(v)$ may be written either in the form of a
Taylor
expansion or a P--Approximant expansion.
The Taylor expansion is only known up to second post--Newtonian order,

\begin{equation}
E^{\prime}(v) = - \eta v \left[1 - \frac{1}{6} (9+\eta) v^{2} -\frac{3}{8}(27-19 \eta +
\frac{\eta^{2}}{3}) v^{4} \right]
\end{equation}

The $n$--PN expression includes terms up to and including $v^{2n}$.
 
The P--Approximant expansion for $E^{\prime}(v)$ is given by

\begin{equation}
E^{\prime}_{P_{n}}(v) =\frac{\eta v}{\left[ 1 + E_{P_{n}} (x) \right] \sqrt{1 + e_{P_{n}} (x)}}
\frac{d e_{P_{n}} (x)}{dx}
\end{equation}
where
\begin{equation}
E_{P_{n}} (x) = \left[ 1 + 2 \eta ( \sqrt{1 + e_{P_{n}} (x)} -1) \right]^{1/2} -1
\end{equation}
and
\begin{equation}
e_{P_{2n}} (x) = -x P^{m}_{m + \epsilon} \left[ \sum^{n}_{k=0} a_{k} x^{k} \right]
\end{equation}
where $x=v^{2}$, and $ P^{m}_{m + \epsilon} \left[ \sum^{n}_{k=0} a_{k} x^{k} \right]$ is the Pade
Approximant of the Taylor series $\sum^{n}_{k=0} a_{k} x^{k}$.
An example of this is
\begin{equation}
e_{P_{4}}(x) = \frac{-x c_{0}}{1 + \frac{c_{1}x}{1 + c_{2}x}} = \frac{-c_{0} x (1 + c_{2}x)}
               {1 + (c_{1} + c_{2}) x}
\end{equation}


The Taylor series expansion of the flux function in the $\eta \neq 0$ case is known up to order
$v^{5}$,
\begin{equation}
\mathcal{F}(v) = \frac{32}{5} \eta^{2} v^{10} \left[ \sum^{5}_{k=0} A_{k} v^{k} \right]
\end{equation}
where
\begin{equation}
A_{0} = 1
\end{equation}
\begin{equation}
A_{1} = 0
\end{equation}
\begin{equation}
A_{2} = - = \frac{1247}{336} - \frac{35}{12} \eta
\end{equation}
\begin{equation}
A_{3} = 4 \pi
\end{equation}
\begin{equation}
A_{4} = - \frac{44711}{9072} + \frac{9271}{504} \eta + \frac{65}{18} \eta^{2}
\end{equation}
\begin{equation}
A_{5} = - \left( \frac{8191}{672} + \frac{535}{24} \eta \right) \pi
\end{equation}
If we now introduce the following "factored" flux function
\begin{equation}
f(v) = \left( 1 - \frac{v}{v_{pole} (\eta) } \right) \,\,  \mathcal{F}(v)
\end{equation}
which we write using Pade Approximants,
\begin{equation}
f_{P_{n}} (v) = \frac{32}{5} \eta^{2} v^{10} P^{m}_{m+\epsilon} \left[ \sum^{5}_{k=0} f_{k} v^{k}
\right]
\end{equation}
where the coefficients $f_{k}$ are defined in terms if the original coefficients $A_{k}$ of the
Taylor
series, $f_{k}=A_{k} - F_{k-1}/v_{pole}$.
Then we re--arrange to define a new $\mathcal{F}(v)$ given by
\begin{equation}
\mathcal{F}_{P_{n}}(v) = \frac{1}{\left( 1 - \frac{v}{v^{P_{n}}_{pole}(\eta)} \right)} \,\,
f_{P_{n}}(v)
\end{equation}
where $v^{P_{n}}_{pole}$ denotes the pole velocity defined by the $v^{n}$ approximant of $e(x)$.
 
This module sets up all the coefficients so that they may be used by the function
\texttt{ChooseModel} to define the energy and flux functions.

\subsubsection*{Algorithm}


\subsubsection*{Uses}
None.

\subsubsection*{Notes}

\vfill{\footnotesize\input{LALInspiralSetupCV}}

</lalLaTeX>  */


   
#include <math.h>
#include <lal/LALInspiral.h>
#include <lal/LALStdlib.h>

static void LALPadeCoeffs7(int n, double *padecoeffs, double *taylorcoeffs);

NRCSID (LALINSPIRALSETUPC, "$Id: LALInspiralSetup.c,v 1.9 2001/12/06 15:15:00 dchurches Exp $");


/*  <lalVerbatim file="LALInspiralSetupCP"> */
void LALInspiralSetup (LALStatus *status,
                       expnCoeffs *ak,
                       InspiralTemplate *params) 
{  /* </lalVerbatim>  */
   
   INT4 ieta;
   /*INT4  pnorder=7;
    * */
   REAL8 lso, eta, vpole;
   REAL8 a1, a2, a3, a4, a5, a6, a7;
   REAL8 c1, c2, c3, c4, c5, c6, c7;
   REAL8 oneby6=1.0/6.0;

   INITSTATUS (status, "LALInspiralSetup", LALINSPIRALSETUPC);

   ASSERT (ak,  status, LALINSPIRALH_ENULL, LALINSPIRALH_MSGENULL);
   ASSERT (params,  status, LALINSPIRALH_ENULL, LALINSPIRALH_MSGENULL);
   ASSERT (params->mass1 > 0, status, LALINSPIRALH_ESIZE, LALINSPIRALH_MSGESIZE);
   ASSERT (params->mass2 > 0, status, LALINSPIRALH_ESIZE, LALINSPIRALH_MSGESIZE);
   ASSERT (params->fLower > 0, status, LALINSPIRALH_ESIZE, LALINSPIRALH_MSGESIZE);
   ASSERT (params->fCutoff > 0, status, LALINSPIRALH_ESIZE, LALINSPIRALH_MSGESIZE);
   ASSERT (params->fCutoff > params->fLower, status, LALINSPIRALH_ESIZE, LALINSPIRALH_MSGESIZE);
   ASSERT (params->tSampling > 0, status, LALINSPIRALH_ESIZE, LALINSPIRALH_MSGESIZE);
   ASSERT (params->tSampling > 2*params->fCutoff, status, LALINSPIRALH_ESIZE, LALINSPIRALH_MSGESIZE);

   vpole = 0.0;
   ak->omegaS = 0.;
   ak->ieta = params->ieta;
   ak->EulerC = 0.577;
   ak->lambda = -(1987./3080. + 3./11. * params->OmegaS);
   ak->theta = params->Theta;
   ak->t0 = params->startTime;
   ak->m1 = params->mass1;
   ak->m2 = params->mass2;
   ak->f0 = params->fLower;
   ak->fn = params->fCutoff;
   ak->samplingrate = params->tSampling;
   ak->samplinginterval = 1./ak->samplingrate;

   lso = sqrt(oneby6);
/* 
   ieta determines the nature of the waveforms: 
   ieta=0 testmass waveforms 
   ieta=1 comparable mass waveforms.
*/
   ieta = ak->ieta;

/* Compute the total mass and eta from m1 and m2 */
   ak->totalmass = (ak->m1 + ak->m2);
   ak->eta = (ak->m1*ak->m2) / (ak->totalmass*ak->totalmass);
   eta = ak->eta;
   ak->totalmass = ak->totalmass * LAL_MTSUN_SI;

/* Set initial velocity according to initial frequency */

   ak->v0 = pow (LAL_PI * ak->totalmass * ak->f0, oneby3);

/* Taylor coefficients of E(x) */
   ak->ETaN = -eta/2.;
   ak->ETa1 = -(9. + ieta*eta)/12.;
   ak->ETa2 = -(27. - 19*ieta*eta + ieta*eta*eta/3.)/8.;
   ak->ETa3 = -675./64. + (209323./4032. - 205.*LAL_PI*LAL_PI/96.
            - 110./9. * ak->lambda)*ieta*eta
            - 155./96. * ieta*eta*eta - 35./5184. * ieta*eta*eta*eta;

/* Taylor coefficients of e(x) */
   ak->eTaN = -1.;
   ak->eTa1 = -1.-ieta*eta/3.;
   ak->eTa2 = -3.+35.*ieta*eta/12.;
   ak->eTa3 = -9.+(26189./504. - 205./96.*LAL_PI*LAL_PI -110./9.*ak->lambda)*ieta*eta
              - 103./36.*ieta*eta*eta + ieta*eta*eta*eta/81.;

/* Taylor coefficients of dE(v)/dv. (NOTE v and NOT x) */
   ak->dETaN = -eta;
   ak->dETa1 = 2.*ak->ETa1;
   ak->dETa2 = 3.*ak->ETa2;
   ak->dETa3 = 4.*ak->ETa3;

/* Pade coefficients of e(x)-function. */
   ak->ePaN = -1.;
   c1 = ak->ePa1 = 1.+ieta*eta/3.;
   c2 = ak->ePa2 = -(144. - 81.*ieta*eta + 4.*ieta*eta*eta) / (36.+12*ieta*eta);
   c3 = ak->ePa3 = (ak->eTa1*ak->eTa3 - ak->eTa2*ak->eTa2)
                 / (ak->eTa1*(ak->eTa1*ak->eTa1 - ak->eTa2));

/* Location of the 0PN and 1PN T- and P-approximant last stable orbit: */
   ak->vlsoT0 = lso;
   ak->vlsoP0 = lso;
   ak->vlsoP2 = lso;
/* 
   vlsoT2 =  6./(9.+ieta*eta); 
   This correct value makes vlso too large for vlsoT2 hence use 1/sqrt(6)
*/
   ak->vlsoT2 = lso;   

/* vlsoT4 is also too large for certain T-approximants: most notably
   TaylorT3, TaylorT2(3PN (1.4,10) crashes)
*/
   ak->vlsoT4 = pow(-ak->ETa1 + pow(ak->ETa1*ak->ETa1 - 3*ak->ETa2,0.5)/(3*ak->ETa2), 0.5);
   ak->vlsoP4 = pow((-1.+pow(-ak->ePa1/ak->ePa2,0.5))/(ak->ePa1 + ak->ePa2), 0.5);
   ak->vpoleP4 = pow(4.*(3.+ieta*eta)/(36.-35.*ieta*eta), 0.5);
/* THE VALUE NEEDS TO BE CHANGED AFTER THE CALCULATION OF CORRECT LSOs */
   ak->vlsoT6 = ak->vlsoT4;
/* 
   vlsoP6 comes from solving a quadratic equation; the plus-root
   is negative and can't be used; the postive root is:
*/
   ak->vlsoP6 = sqrt(0.5/(c2*c2+c3*c3+c2*c1+2.*c2*c3)
              *(-2.*c3 - 2.*c2 - 2.*sqrt(-c2*c1)));
/* The 3PN pole doesn't exist for eta=1/4. So decided to use 2PN pole always */
   ak->vpoleP6 = ak->vpoleP4;
/*
   a = c1*c3;
   b = c1+c2+c3;
   vpole = (-b + sqrt(b*b-4.*a))/(2.*a); 
   fprintf(stderr, "%e %e %e %e %e vpolePlus=%e\n", c1, c2, c3, b*b, 4.*a, vpole);
   vpole = (-b - sqrt(b*b-4.*a))/(2.*a); 
   fprintf(stderr, "%e %e vpoleMinus=%e\n", ak->vlsoP4, ak->vlsoP6, vpole);
   ak->vpoleP6 = vpole;
*/

/* Taylor coefficients of flux. */
   ak->fTaN = ak->fPaN = ak->FTaN = 32.*eta*eta/5.;
   ak->FTa1 = 0.;
   ak->FTa2 = -1247./336.-35.*ieta*eta/12.;
   ak->FTa3 = 4.*LAL_PI;
   ak->FTa4 = -44711./9072.+9271.*ieta*eta/504.+65.*ieta*eta*eta/18.;
   ak->FTa5 = -(8191./672.+535./24.*ieta*eta)*LAL_PI;
   ak->FTa6 = 6643739519./69854400. + 16.*LAL_PI*LAL_PI/3.
            - 1712./105.*ak->EulerC+ (-11497453./272160. + 41.*LAL_PI*LAL_PI/48.
               + 176./9. * ak->lambda - 88.*ak->theta/3.) * ieta*eta
            - 94403./3024. * ieta*eta*eta - 775./324. * ieta * eta*eta*eta;
   ak->FTl6 = -1712./105.;
   ak->FTa7 = LAL_PI * (-16285./504. + 176419./1512. * ieta*eta
               + 19897./378.* ieta*eta*eta);

/* Initialize members of the structures which get fed into 
   phasing3() and frequency3(). 
*/

  ak->ptaN = -2./eta;
  ak->pta2 = 3715./8064. + 55.*ieta*eta/96.;
  ak->pta3 = -0.75*LAL_PI;
  ak->pta4 = 9.275495/14.450688 + 2.84875*ieta*eta/2.58048 
           + 1855.*ieta*eta*eta/2048.;
  ak->pta5 = -(3.8645/17.2032 + 15./2048.*ieta*eta) * LAL_PI;
  ak->pta6 =  (83.1032450749357/5.6782522275840 - 53./40.*LAL_PI*LAL_PI 
           - 107./56. * ak->EulerC + (-123.292747421/4.161798144 
           + 2.255/2.048 *LAL_PI*LAL_PI + 385./48. * ak->lambda 
           - 55./16.* ak->theta) * ieta * eta + 1.54565/18.35008 * ieta*eta*eta 
           - 1.179625/1.769472 * ieta*eta*eta*eta);

  ak->pta7 =  (1.88516689/1.73408256 + 1.40495/1.14688 * ieta*eta
           - 1.22659/5.16096 * ieta*eta*eta) * LAL_PI;
  ak->ptl6 = 107./448.;

  ak->ftaN = 1./(8.*LAL_PI*ak->totalmass);
  ak->fta2 = 743./2688.+(11.*ieta*eta)/32.;
  ak->fta3 = -0.3*LAL_PI;
  ak->fta4 = 1.855099/14.450688 +  5.6975*ieta*eta/25.8048 
           + 3.71*ieta*eta*eta/20.48;
  ak->fta5 = -(7.729/21.504 + 3./256.*ieta*eta) * LAL_PI;
  ak->fta6 = (-7.20817631400877/2.88412611379200 + (53./200.) * LAL_PI*LAL_PI
           + 1.07/2.80 * ak->EulerC
           + (1.23292747421/.20808990720 - 4.51/20.48 * LAL_PI*LAL_PI
           - 77./48. * ak->lambda + 11./16. * ak->theta) * ieta*eta
           - 3.0913/183.5008 * ieta*eta*eta 
           + 2.35925/17.69472 * ieta*eta*eta*eta);

  ak->fta7 = (-1.88516689/4.33520640 - 2.8099/5.7344 * ieta*eta
           + 1.22659/12.90240 * ieta*eta*eta)*LAL_PI;

/* Initialize members of the structures which get fed into 
   phasing2() and frequency2(). 
*/

  ak->tvaN = -5.*ak->totalmass/(256.*eta);
  ak->tva2 = (743./252. + 11./3. * ieta * eta);
  ak->tva3 = -32./5. * LAL_PI;
  ak->tva4 = 30.58673/5.08032 + 54.29/5.04*ieta*eta + 61.7/7.2*ieta*eta*eta;
  ak->tva5 = -(77.29/2.52 + ieta*eta) * LAL_PI;
  ak->tva6 = - 1005.2469856691/2.3471078400 + 136.96/1.05*log(2.) 
           + (128./3.- 451./12.*ieta*eta) * LAL_PI*LAL_PI 
           + 68.48/1.05*ak->EulerC
           + 1533.5597827/1.5240960 * ieta*eta
           - 15.211/1.728*ieta*eta*eta 
           + 25.565/1.296*ieta*eta*eta*eta 
           + 352./3.*ieta*eta*ak->theta
           - 2464./9.*ieta*eta*ak->lambda;
  ak->tva7 =  -154.19335/1.27008*LAL_PI + 7373./189.*ieta*eta*eta*LAL_PI
           - 202.823/1.512*LAL_PI*eta;
  ak->tvl6 = 68.48/3.15;

  ak->pvaN = -1./(16.*eta);
  ak->pva2 = (3715./1008. + 55./12.*ieta*eta);
  ak->pva3 = -10.*LAL_PI;
  ak->pva4 = (15293365./1016064. + 27145./1008.*ieta*eta 
             + 3085./144.*ieta*eta*eta);
  ak->pva5 = (38645./672. + 15./8. * ieta*eta) * LAL_PI;
  ak->pva6 = 1234.8611926451/1.8776862720 - 160./3. * LAL_PI*LAL_PI 
           - 1712./21. * ak->EulerC
           + (-15335597827./12192768. + 2255./48. * LAL_PI*LAL_PI 
           + 3080./9.*ak->lambda - 440./3. * ak->theta) * ieta*eta  
           + 76055./6912. * ieta*eta*eta - 127825./5184. * ieta*eta*eta*eta;
  ak->pva7 = (77096675./2032128. + 1014115./24192.*ieta*eta 
           - 36865./3024.*ieta*eta*eta) * LAL_PI;
  ak->pvl6 = - 1712./21.;

/* 
  Taylor coefficients of f(v)=(1-v/vpole)F(v) 
*/
 
   switch (params->order)
   {
      case newtonian:
      case onePN:
      case onePointFivePN:
      case twoPN:
      case twoPointFivePN:
         vpole = ak->vpoleP4;
         break;
      case threePN:
      case threePointFivePN:
         vpole = ak->vpoleP6;
         break;
   }

   ak->fTa1 = ak->FTa1 - 1./vpole;
   ak->fTa2 = ak->FTa2 - ak->FTa1/vpole;
   ak->fTa3 = ak->FTa3 - ak->FTa2/vpole;
   ak->fTa4 = ak->FTa4 - ak->FTa3/vpole;
   ak->fTa5 = ak->FTa5 - ak->FTa4/vpole;
   ak->fTa6 = ak->FTa6 - ak->FTa5/vpole;
   ak->fTa7 = ak->FTa7 - ak->FTa6/vpole;

/* 
   Pade coefficients of f(v);  assumes that a0=1 => c0=1 
*/

   a1 = ak->fTa1;
   a2 = ak->fTa2;
   a3 = ak->fTa3;
   a4 = ak->fTa4;
   a5 = ak->fTa5;
   a6 = ak->fTa6;
   a7 = ak->fTa7;

   c1 = -a1;
   c2 = -(c1*c1 - a2)/c1;
   c3 = -(c1*pow(c2+c1,2.) + a3)/(c1*c2);
   c4 = -(c1*pow(c2+c1,3.) + c1*c2*c3*(c3+2*c2+2*c1) - a4)/(c1*c2*c3);
   c5 = -(c1*pow(pow(c1+c2,2.)+c2*c3,2.) + c1*c2*c3*pow(c1+c2+c3+c4,2.) + a5)
      /(c1*c2*c3*c4);
   c6 = -(c1*pow(c1+c2, 5.) 
      + c1*c2*c3*(pow(c1+c2+c3,3.) + 3.*pow(c1+c2,3.) 
      + 3.*c2*c3*(c1+c2) + c3*c3*(c2-c1)) 
      + c1*c2*c3*c4*(pow(c1+c2+c3+c4,2.) 
      + 2.*pow(c1+c2+c3,2.) + c3*(c4-2.*c1)) 
      + c1*c2*c3*c4*c5*(c5 + 2.*(c1+c2+c3+c4)) 
      - a6)/(c1*c2*c3*c4*c5);

   c7 = -(15.*pow(c1,5.)*pow(c2,2.)
      + 6.*pow(c1,2.)*c2*pow(c3,3.)*c4
      + 6.*c1*c2*pow(c3,3.)*pow(c4,2.)
      + a7
      + c1*pow(c2,6.)
      + 2.*c1*c2*pow(c3,2.)*c4*c5*c6
      + 3.*c1*c2*pow(c4,2.)*pow(c5,2.)*c3
      + c1*c2*c3*c4*c5*pow(c6,2.)
      + 6.*c1*c2*pow(c3,2.)*pow(c4,2.)*c5
      + 2.*c1*c2*c3*pow(c4,2.)*c5*c6
      + 3.*c1*c2*pow(c3,3.)*c4*c5
      + 3.*c1*c2*pow(c4,3.)*c5*c3
      + c1*c2*c3*c4*pow(c5,3.)
      + 4.*pow(c2,2.)*c1*c3*pow(c4,2.)*c5
      + 2.*pow(c2,2.)*c1*c3*c4*c5*c6
      + 5.*c1*pow(c2,2.)*pow(c3,4.)
      + 10.*c1*pow(c2,3.)*pow(c3,3.)
      + 10.*c1*pow(c2,4.)*pow(c3,2.)
      + 5.*pow(c2,5.)*c3*c1
      + pow(c1,7.)
      + 4.*pow(c1,4.)*c2*c3*c4
      + 18.*pow(c1,2.)*pow(c2,2.)*pow(c3,2.)*c4
      + 12.*pow(c1,2.)*pow(c2,3.)*c3*c4
      + 12.*pow(c1,3.)*pow(c2,2.)*c3*c4
      + 15.*pow(c1,3.)*pow(c2,4.)
      + 12.*pow(c1,2.)*pow(c2,2.)*pow(c3,3.)
      + 24.*pow(c1,2.)*pow(c2,3.)*pow(c3,2.)
      + 20.*pow(c1,2.)*pow(c2,4.)*c3
      + 18.*pow(c1,3.)*pow(c2,2.)*pow(c3,2.)
      + 30.*pow(c1,3.)*pow(c2,3.)*c3
      + 6.*pow(c1,2.)*pow(c2,5.)
      + 4.*pow(c1,4.)*c2*pow(c3,2.)
      + 20.*pow(c1,4.)*pow(c2,2.)*c3
      + c1*c2*pow(c3,5.)
      + 20.*pow(c1,4.)*pow(c2,3.)
      + 6.*c2*pow(c1,6.)
      + 5.*pow(c1,5.)*c2*c3
      + 3.*pow(c1,3.)*c2*c3*pow(c4,2.)
      + 6.*pow(c1,3.)*c2*pow(c3,2.)*c4
      + 3.*pow(c1,3.)*c2*pow(c3,3.)
      + 3.*pow(c1,3.)*c2*c3*c4*c5
      + 2.*pow(c1,2.)*c2*c3*c4*pow(c5,2.)
      + 6.*pow(c1,2.)*c2*pow(c3,2.)*pow(c4,2.)
      + 2.*pow(c1,2.)*c2*c3*pow(c4,3.)
      + 4.*pow(c1,2.)*c2*pow(c3,2.)*c4*c5
      + 4.*pow(c1,2.)*c2*c3*pow(c4,2.)*c5
      + 6.*c1*pow(c2,2.)*pow(c3,2.)*c4*c5
      + 2.*pow(c1,2.)*c2*c3*c4*c5*c6
      + 6.*pow(c1,2.)*pow(c2,2.)*c3*c4*c5
      + 2.*pow(c1,2.)*c2*pow(c3,4.)
      + 6.*pow(c1,2.)*pow(c2,2.)*c3*pow(c4,2.)
      + 2.*c1*c2*pow(c3,2.)*c4*pow(c5,2.)
      + 3.*c1*pow(c2,3.)*c3*c4*c5
      + 9.*c1*pow(c2,2.)*pow(c3,2.)*pow(c4,2.)
      + 12.*c1*pow(c2,2.)*pow(c3,3.)*c4
      + 3.*c1*pow(c2,3.)*c3*pow(c4,2.)
      + 2.*c1*c2*c3*c4*pow(c5,2.)*c6
      + 12.*c1*pow(c2,3.)*pow(c3,2.)*c4
      + c1*c2*c3*pow(c4,4.)
      + 4.*c1*c2*pow(c3,2.)*pow(c4,3.)
      + 4.*c1*pow(c3,4.)*c4*c2
      + 4.*c1*pow(c2,4.)*c3*c4
      + 2.*pow(c2,2.)*c1*c3*pow(c4,3.)
      + 2.*pow(c2,2.)*c1*c3*c4*pow(c5,2.))/(c1*c2*c3*c4*c5*c6);

   ak->fPa1 = c1;
   ak->fPa2 = c2;
   ak->fPa3 = c3;
   ak->fPa4 = c4;
   ak->fPa5 = c5;
   ak->fPa6 = c6;
   ak->fPa7 = c7;

/*
   taylorcoeffs[0] = 1.;
   taylorcoeffs[1] = a1;
   taylorcoeffs[2] = a2;
   taylorcoeffs[3] = a3;
   taylorcoeffs[4] = a4;
   taylorcoeffs[5] = a5;
   taylorcoeffs[6] = a6;
   taylorcoeffs[7] = a7;
   LALPadeCoeffs7(pnorder, padecoeffs, taylorcoeffs);
   fprintf(stderr, "%e %e %e %e %e %e %e\n", a1, a2, a3, a4, a5, a6, a7);
   fprintf(stderr, "%e %e %e %e %e %e %e\n", c1, c2, c3, c4, c5, c6, c7);
   fprintf(stderr, "%e %e %e %e %e %e %e\n", padecoeffs[1], padecoeffs[2], 
   padecoeffs[3], padecoeffs[4], padecoeffs[5], padecoeffs[6], padecoeffs[7]);

*/
   RETURN (status);

}

/* pade_coeffs.f -- translated by f2c (version 20000531).  */

/* ---------------------------------------------------------------------- */
/* {\sf pade\_coeffs (int n, real*8 as(0:n), real*8 cs(0:n))} */
/* ---------------------------------------------------------------------- */
/* 	First Version: 4.1.97. */
/* 	Purpose: Given Taylor coefficients as(0)...as(n) (n<=11) to */
/* 	        find (near) diagonal Pade coefficients cc(0)...cc(n) of the */
/*             continued fraction form at various orders. */
/* 	AUTHORS: T.Damour, B.R. Iyer and B.S.Sathyaprakash. */
/* 	Revision History: Last Update:23.6.97. */
/* 	DEPENDENCIES: None. */
/* 	INPUTS: */
/* 	   n: Array size of Taylor coefficients. */
/*           At the moment n can atmost be 11. */
/* 	   as: Array contianining the Taylor coefficients. */
/* 	OUTPUTS: */
/* 	   cs: Array contianining the Pade coefficients. */
/* 	NOTES: Damour, Iyer and Sathyaprakash (1997). */
/* ---------------------------------------------------------------------- */
/* Subroutine */ 
static void LALPadeCoeffs7(int n, double *cs, double *as)
{
    /* System generated locals */
    int i__1;
    double d__1;

    /* Local variables */
    static int i__;
    static double a0, a1, c0, c1, a2, c2, a3, c3, a4, c4, a5, 
	    c5, a6, c6, a7;
    
    static double t3, t4, t8, t9, t10, t11, t12, t13, t14, t15, t21, t18, t19,
	    t22, t23, t24, t25, t26, t27, t28, t29, t30, t32, t33, t34, t37, 
	    t38, t40, t41, t42, t43, t44, t45, t46, t47, t48, t50, t51, t52, 
	    t53, t54, t55, t56, t57, t58, t59, t60, t61, t62, t63, t64, t65, 
	    t66, t67, t68, t69, t70, t71, t72, t73, t76, t77, t78, t80, t81, 
	    t82, t83, t84, t85, t86, t87, t88, t89, t90, t91, t92, t93, t95, 
	    t96, t97, t98, t99, 
            t100, t101, t103, t104, t106, t107, t108, 
	    t109, t110, t111, t112, t113, t114, t115, t116, t117, t118, t119, 
	    t120, t121, t123, t124, t125, t126, t127, t128, t130, t131, t133, 
	    t134, t135, t136, t137, t138, t139, t140, t144, t145, t148, t150, 
	    t151, t153, t155, t156, t157, t158, t160, t161, t162, t163, t164, 
	    t165, t167, t168, t170, t171, t173, t174, t176, t177, t179, t181, 
	    t182, t185, t188, t190, t191, t192, t193, t194, t196, t199, t201, 
	    t202, t203, t205, t207, t210, t212, t214, t216, t217, t219, t229, 
	    t231, t233, t234, t235, t236, t237, t238, t240, t243, t244, t245, 
	    t247, t248, t249, t250, t251, t252, t254, t255, t260, t261, t262, 
	    t265, t267, t268, t270, t271, t273, t275, t276, t278, t280, t281, 
	    t283, t284, t286, t289;

/* ---------------------------------------------------------------------- */
    a0 = as[0];
    i__1 = n;
    for (i__ = 0; i__ <= i__1; ++i__) {
	cs[i__] = 0.;
    }
    cs[0] = a0;
    if (n == 0 || cs[0] == 0.) {
	return ;
    }
    c0 = cs[0];
    a1 = as[1];
    t3 = 1 / c0;
    cs[1] = -a1 / a0;
    if (n == 1 || cs[1] == 0.) {
	return ;
    }
    c1 = cs[1];
    a2 = as[2];
    t4 = 1 / c1;
    t8 = c0 * c1;
    cs[2] = t3 * t4 * a2 - c1;
    if (n == 2 || cs[2] == 0.) {
	return ;
    }
    c2 = cs[2];
    a3 = as[3];
/* Computing 2nd power */
    d__1 = c2;
    t9 = d__1 * d__1;
    t10 = t8 * t9;
/* Computing 2nd power */
    d__1 = c1;
    t11 = d__1 * d__1;
    t12 = c0 * t11;
    t13 = t12 * c2;
    t14 = t11 * c1;
    t15 = c0 * t14;
    t18 = 1 / c2;
    t19 = t4 * t18;
    cs[3] = -(a3 + t10 + t13 * 2 + t15) * t3 * t19;
    if (n == 3 || cs[3] == 0.) {
	return ;
    }
    c3 = cs[3];
    a4 = as[4];
/* Computing 2nd power */
    d__1 = c3;
    t21 = d__1 * d__1;
    t22 = c2 * t21;
    t23 = t8 * t22;
    t24 = t9 * c3;
    t25 = t8 * t24;
    t26 = t9 * c2;
    t27 = t8 * t26;
    t28 = c2 * c3;
    t29 = t12 * t28;
    t30 = t12 * t9;
    t32 = t14 * c2 * c0;
/* Computing 2nd power */
    d__1 = t11;
    t33 = d__1 * d__1;
    t34 = c0 * t33;
    t37 = 1 / c3;
    t38 = t19 * t37;
    t40 = t11 * t26;
    t41 = t40 * c0;
/* Computing 2nd power */
    d__1 = t9;
    t42 = d__1 * d__1;
    t43 = t8 * t42;
    t44 = t11 * t9;
    t45 = c0 * c3;
    t46 = t44 * t45;
    t47 = t14 * t9;
    t48 = t47 * c0;
    t50 = t33 * c2 * c0;
    t51 = t33 * c1;
    t52 = c0 * t51;
    t53 = t15 * t28;
    cs[4] = -(-a4 + t23 + t25 * 2 + t27 + t29 * 2 + t30 * 3 + t32 * 3 + t34) *
	     t3 * t38;
    if (n == 4 || cs[4] == 0.) {
	return ;
    }
    c4 = cs[4];
    a5 = as[5];
    t54 = t28 * c4;
    t55 = t12 * t54;
    t56 = t12 * t22;
    t57 = c0 * t9;
    t58 = t21 * c1;
    t59 = t57 * t58;
    t60 = c0 * t26;
    t61 = c3 * c1;
    t62 = t60 * t61;
    t63 = t24 * c4;
    t64 = t8 * t63;
/* Computing 2nd power */
    d__1 = c4;
    t65 = d__1 * d__1;
    t66 = t28 * t65;
    t67 = t8 * t66;
    t68 = t22 * c4;
    t69 = t8 * t68;
    t70 = t21 * c3;
    t71 = c2 * t70;
    t72 = t8 * t71;
    t73 = a5 + t41 * 4 + t43 + t46 * 6 + t48 * 6 + t50 * 4 + t52 + t53 * 3 + 
	    t55 * 2 + t56 * 2 + t59 * 3 + t62 * 3 + t64 * 2 + t67 + t69 * 2 + 
	    t72;
    t76 = t18 * t37;
    t77 = 1 / c4;
    t78 = t76 * t77;
    t80 = t21 * t65;
    t81 = t80 * c2;
    t82 = t8 * t81;
    t83 = t26 * c3;
    t84 = t12 * t83;
    t85 = t9 * t21;
    t86 = t85 * c4;
    t87 = t8 * t86;
    t88 = c0 * t42;
    t89 = t88 * t61;
    t90 = t26 * t21;
    t91 = t8 * t90;
    t92 = c3 * c4;
    cs[5] = -t73 * t3 * t4 * t78;
    if (n == 5 || cs[5] == 0.) {
	return ;
    }
    c5 = cs[5];
    a6 = as[6];
    t93 = t92 * c5;
    t95 = t34 * t28;
    t96 = t65 * c4;
    t97 = t28 * t96;
    t98 = t8 * t97;
    t99 = t8 * c2;
/* Computing 2nd power */
    d__1 = c5;
    t100 = d__1 * d__1;
    t101 = t92 * t100;
    t103 = t21 * c4;
    t104 = t103 * c5;
    t106 = t70 * c4;
    t107 = t106 * c2;
    t108 = t8 * t107;
    t109 = t12 * t85;
    t110 = t15 * t22;
    t111 = t15 * t24;
    t112 = t12 * t66;
    t113 = t82 * -3 + a6 - t84 * 12 - t87 * 6 - t89 * 4 - t91 * 6 - t13 * 2 * 
	    t93 - t95 * 4 - t98 - t99 * t101 - t99 * 2 * t104 - t108 * 3 - 
	    t109 * 9 - t110 * 3 - t111 * 12 - t112 * 2;
    t114 = t24 * t65;
    t115 = t8 * t114;
    t116 = t42 * c2;
    t117 = t8 * t116;
    t118 = t15 * t54;
    t119 = t33 * t11;
    t120 = c0 * t119;
    t121 = t12 * t63;
    t123 = t51 * c2 * c0;
    t124 = t15 * t26;
/* Computing 2nd power */
    d__1 = t21;
    t125 = d__1 * d__1;
    t126 = c2 * t125;
    t127 = t8 * t126;
    t128 = t34 * t9;
    t130 = c3 * t65;
    t131 = t130 * c5;
    t133 = t12 * t42;
    t134 = t83 * c4;
    t135 = t8 * t134;
    t136 = t12 * t68;
    t137 = t9 * t70;
    t138 = t8 * t137;
    t139 = t12 * t71;
    t140 = t115 * -2 - t117 - t118 * 3 - t120 - t121 * 6 - t123 * 5 - t124 * 
	    10 - t127 - t128 * 10 - t10 * 2 * t93 - t99 * 2 * t131 - t133 * 5 
	    - t135 * 3 - t136 * 4 - t138 * 4 - t139 * 2;
    t144 = 1 / c5;
    t145 = t77 * t144;
    t148 = t106 * c5;
    t150 = t125 * c4;
    t151 = t150 * c2;
    t153 = t80 * c5;
    t155 = t26 * t70;
    t156 = t8 * t155;
    cs[6] = (t113 + t140) * t3 * t4 * t76 * t145;
    if (n == 6 || cs[6] == 0.) {
	return ;
    }
    c6 = cs[6];
    a7 = as[7];
    t157 = c5 * c6;
    t158 = t130 * t157;
    t160 = t125 * c3;
    t161 = c2 * t160;
    t162 = t8 * t161;
/* Computing 2nd power */
    d__1 = c6;
    t163 = d__1 * d__1;
    t164 = c5 * t163;
    t165 = t92 * t164;
    t167 = t65 * t100;
    t168 = t167 * c3;
    t170 = t96 * c5;
    t171 = t170 * c3;
    t173 = t100 * c6;
    t174 = t92 * t173;
    t176 = t100 * c5;
    t177 = t92 * t176;
    t179 = t92 * t157;
/* Computing 2nd power */
    d__1 = t65;
    t181 = d__1 * d__1;
    t182 = t28 * t181;
    t185 = t24 * t96;
    t188 = t99 * 3 * t148 + t8 * 4 * t151 + t99 * 6 * t153 + t156 * 10 + t99 *
	     2 * t158 + t162 + t99 * t165 + t99 * 3 * t168 + t99 * 3 * t171 + 
	    t99 * 2 * t174 + t99 * t177 + t10 * 2 * t179 + t8 * t182 + t10 * 
	    2 * t101 + t8 * 2 * t185 + t10 * 4 * t131;
    t190 = t42 * t21;
    t191 = t8 * t190;
    t192 = c0 * t116;
    t193 = t192 * t61;
    t194 = t85 * t65;
    t196 = t137 * c4;
    t199 = t83 * t65;
    t201 = t9 * t125;
    t202 = t8 * t201;
    t203 = t90 * c4;
    t205 = t22 * t96;
    t207 = t71 * t65;
    t210 = t52 * t28;
    t212 = t15 * t71;
    t214 = t10 * 6 * t104 + t191 * 10 + t193 * 5 + t8 * 9 * t194 + t8 * 12 * 
	    t196 + t27 * 3 * t93 + t8 * 3 * t199 + t202 * 5 + t8 * 12 * t203 
	    + t8 * 4 * t205 + t8 * 6 * t207 + t32 * 3 * t93 + t210 * 5 + t15 *
	     3 * t66 + t212 * 3 + t15 * 6 * t68;
    t216 = t42 * c3;
    t217 = t216 * c4;
    t219 = t12 * t126;
    t229 = t103 * t157;
    t231 = t103 * t100;
    t233 = t34 * t26;
    t234 = t15 * t42;
    t235 = t12 * t116;
    t236 = t8 * 4 * t217 + t219 * 2 + t30 * 6 * t93 + t12 * 6 * t81 + t12 * 6 
	    * t114 + t12 * 6 * t107 + t13 * 4 * t104 + t13 * 2 * t179 + t13 * 
	    2 * t101 + t13 * 4 * t131 + t12 * 2 * t97 + t99 * 2 * t229 + t99 *
	     2 * t231 + t233 * 20 + t234 * 15 + t235 * 6;
    t237 = t12 * t216;
    t238 = t15 * t85;
    t240 = t12 * t90;
    t243 = t33 * t14;
    t244 = c0 * t243;
    t245 = t34 * t24;
    t247 = t15 * t83;
    t248 = t34 * t22;
    t249 = t12 * t137;
    t250 = t42 * t9;
    t251 = t8 * t250;
    t252 = t52 * t9;
    t254 = t119 * c2 * c0;
    t255 = t237 * 20 + t238 * 18 + t12 * 18 * t86 + t240 * 24 + t15 * 12 * 
	    t63 + t12 * 12 * t134 + t244 + t245 * 20 + t34 * 4 * t54 + a7 + 
	    t247 * 30 + t248 * 4 + t249 * 12 + t251 + t252 * 15 + t254 * 6;
    t260 = t37 * t77;
    t261 = 1 / c6;
    t262 = t144 * t261;
    t265 = t12 * t250;
    t267 = t243 * c2 * c0;
    t268 = t71 * t96;
    t270 = t181 * c4;
    t271 = t28 * t270;
    t273 = t126 * t65;
    t275 = t160 * c4;
    t276 = t275 * c2;
    t278 = t106 * t157;
    t280 = t21 * t96;
    t281 = t280 * c5;
    t283 = t70 * t65;
    t284 = t283 * c5;
    t286 = t106 * t100;
    t289 = c4 * c5;
    cs[7] = -(t188 + t214 + t236 + t255) * t3 * t19 * t260 * t262;
    return ;
} /* LALPadeCoeffs7 */
